#! /usr/bin/perl -w
#
# REST API Demo -- "write network data"
# http://www.infoblox.com/community
#
# This program shows some basic aspects of interacting with the infoblox WAPI (REST-based Web API) for NIOS.
#
# 1) set up authentication with the system
# 2) query the system to discover defined networks
# 3) query the system to get the information about each network and display it
#
#

#
# These libraries may not be installed on your system by default. To install them, you can use CPAN
#
# % sudo perl -e shell -MCPAN
#
# with the install command (for example, 'install WWW::Curl'
use WWW::Curl::Easy;
use JSON;
use Data::Dumper;

#
# authentication configuration
#
# this script hard wires access to your NIOS server. This might not be what you want to do
# in a production script. You need to know the IP address of your grid server, and an account and
# password with access to the system.
#
my $serveraddr = "10.65.16.79";
my $acct = "admin";
my $passwd = "infoblox";

#
# set up the base REST API url
my $curl_base = "https://" . $serveraddr . "/wapi/v1.0/";
#
# since we're only using the network operation, we wire it in here
my $curlcmd = $curl_base . "network";

#
# The data to send to create the network
#my $networkinfo = "network=10.4.0.0/16&_return_type=xml-pretty";
my $networkinfo = "network=10.6.0.0/16";

#
# To interact with the WAPI, we're going to use the WWW:Curl library. We're using this instead
# of LWP because there are some bugs in some versions of perl when using LWP with HTTPS/SSL that
# made writing the demo code in a readable way difficult. Either technique can work well;
# WWW::Curl is easier and you can so some of your testing using the CLI curl command, but
# LWP is more powerful and flexible (but more complicated).

#
# set up the curl object for use
my $curl = WWW::Curl::Easy->new;
#$curl->setopt(CURLOPT_VERBOSE,1);  # turns on debug mode

#
# if you're working in a lab, chances are good you're interacting with a machine running SSL, but
# without a valid security certificate (if you are someone who installs certificates on lab machines,
# congratulations). To disable certificate checking, you set CURLOPT_SSL_VERIFYPEER to 0
# (the equivalent of the curl -k option). Note: you should never do this outside of a lab
# environment, since it disconnects the security validations of SSL.
$curl->setopt(CURLOPT_SSL_VERIFYPEER, 0); # disables SSL cert checking. For lab use only. seriously!

#
# This is where we set the authentication information for the server (equivalent to the curl -u
# option). These will stay active with the curl object we've created until we destroy it or
# reset them, so you can set this once and make multiple requests against the server without
# having to supply them each time.
$curl->setopt(CURLOPT_USERPWD, "$acct:$passwd");

#
# Format the URL to interact with the REST API. This URL is set up to do a GET of the networks configured into
# the system, and will bring back all of the data items defined in that big blob of text above. The
# general form of this is:
#
# https://<serveraddr>/network/
#
$curl->setopt(CURLOPT_URL, $curlcmd);

#
# this sets up the location where the call to the server will store its data. We're creating
# a variable, then passing a reference to the server.
my $response_body;
$curl->setopt(CURLOPT_WRITEDATA, \$response_body); # sets up the location for the returned data to go.

#
# set up the command as a post and load the data
$curl->setopt(CURLOPT_POST, 1);
$curl->setopt(CURLOPT_POSTFIELDS, $networkinfo);

#
# and now, actually execute the request.
my $retcode = $curl->perform;
#print "\$retcode = $retcode\n";

# we need to get the returned HTTP code. If that's between 400 and 599, we code
# some kind of error on our API request, so flag it as an error.
#
# This may need some explanation. If $retcode returns non-zero, the attempt to
# execute the API call failed in some way. if $retcode returns zero, then the
# $http_code returned will tell you whether the request contained in the API call
# succeeded or failed in some way. If you think about this in terms of a browser,
# $retcode returning non-zero is like typing in a URL and having the browser crash,
# where $http_code is typing in the URL and having the browser return
# something like a "404: page not found".
my $http_code = $curl->getinfo(CURLINFO_HTTP_CODE);
my $http_error = 0;
$http_error = $http_code if ($http_code >= 400 && $http_code < 600) ;
#print "\$http_code = $http_code, \$http_error = $http_error\n";

#
# curl->perform returns 0 if the operation works, and a non-zero value if
# there's a problem.
#
# you then need to check the HTTP return code. If it's between 400 and 599, you had
# an error, which means that the API call succeeded but the actuall request failed
# for some reason. If either happens, you need to process via your error code.
#
# if there's a problem, the error information is returned within $response_body
# as a JSON packet. note we return JSON even if you're requested the data in
# another form such as XML, which will complicate your processing somewhat depending
# on what you want to do with errors. In this case, I use Data::Dumper to format
# it and display it as we exit.
if ($retcode == 0 && $http_error == 0) {

#
# the data returned by this if the network creation succeeded is a network identifier
# which will look something like this:
#
# "network/ZG5zLm5ldHdvcmskMTAuNC4wLjAvMTYvMA:10.4.0.0/16/default"
#
# it is NOT in JSON or XML format, so it doesn't need to be decoded. In this case, since we
# aren't doing anything with it, we'll just display it and exit. For a more complex example,
# we should capture this and then use it to identify the network in other API requests.

	print Dumper($response_body);

#
# note: if you run this script and the network already exists (try running the script twice!),
# you'll get an error. That's the expected functionality, but it can surprise you the first
# time it happens.

} else {
#
#	Something bad happened. The error data is returned withing the $response_body. This will be returned in JSON even if
#	you've requested data in XML or some other format, so be ready for that if you intend to pull apart and use this
#	data to manage errors. For the purposes of this example, we'll just display it and exit.
	print "Operation failed: "; print "\$retcode = $retcode, \$http_error = $http_error\n";
	my $decoded_response = from_json($response_body);
	print Dumper($decoded_response);
}

exit;
